import copy
from datetime import datetime

from django import forms
from django.utils import timezone


class ParsedResponse(object):
    def __init__(self, parsed_data, is_valid, errors):
        self.parsed_data = parsed_data
        self.is_valid = is_valid
        self.errors = errors


class ListField(forms.MultiValueField):
    def clean(self, value):
        if hasattr(value, '__len__'):
            self.fields = [forms.CharField(required=False)
                           for _ in range(len(value))]
        return super(ListField, self).clean(value)

    def compress(self, data_list):
        return data_list


class TokenInformationForm(forms.Form):
    user_id = forms.CharField()
    token = forms.CharField()
    expires_at = forms.CharField()
    token_is_valid = forms.BooleanField()
    scopes = ListField(required=False)

    def __init__(self, initial, *args, **kwargs):
        super(TokenInformationForm, self).__init__(initial, *args, **kwargs)
        initial['token_is_valid'] = initial.get('is_valid', False)

    def clean_token_is_valid(self):
        if 'token_is_valid' not in self.cleaned_data:
            raise forms.ValidationError('No token status in response.')
        is_valid = self.cleaned_data['token_is_valid']
        if not is_valid:
            raise forms.ValidationError('Token is invalid.')
        return is_valid

    def clean_expires_at(self):
        timestamp = self.data['expires_at']
        naive = datetime.fromtimestamp(int(timestamp))
        return naive.replace(tzinfo=timezone.utc)


class FacebookResponseError(Exception):
    def __init__(self, errors):
        super(FacebookResponseError, self).__init__()
        self.errors = errors


def parse_facebook_response(raw_response, token):
    try:
        return try_to_parse_facebook_response(raw_response, token)
    except FacebookResponseError as e:
        return ParsedResponse(None, False, e.errors)


def try_to_parse_facebook_response(raw_response, token):
    if not isinstance(raw_response, dict):
        raise FacebookResponseError(['Facebook response should be dict.'])
    data = copy.deepcopy(raw_response.get('data', {}))
    if not isinstance(data, dict):
        raise FacebookResponseError(['Facebook data should be dict.'])
    data['token'] = token
    form = TokenInformationForm(data)
    if form.is_valid():
        return ParsedResponse(form.cleaned_data, True, None)
    else:
        raise FacebookResponseError(form.errors)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration
from django.utils import timezone


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'UserToken.granted_at'
        db.add_column('facebook_auth_usertoken', 'granted_at',
                      self.gf('django.db.models.fields.DateTimeField')(blank=True, auto_now_add=True, default=timezone.now,
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'UserToken.granted_at'
        db.delete_column('facebook_auth_usertoken', 'granted_at')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '80', 'unique': 'True'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True', 'symmetrical': 'False'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'blank': 'True', 'max_length': '75'}),
            'first_name': ('django.db.models.fields.CharField', [], {'blank': 'True', 'max_length': '30'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'related_name': "'user_set'", 'blank': 'True', 'symmetrical': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'blank': 'True', 'max_length': '30'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'related_name': "'user_set'", 'blank': 'True', 'symmetrical': 'False'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'ordering': "('name',)", 'db_table': "'django_content_type'", 'object_name': 'ContentType'},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'facebook_auth.facebookuser': {
            'Meta': {'_ormbases': ['auth.User'], 'object_name': 'FacebookUser'},
            'app_friends': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['facebook_auth.FacebookUser']", 'related_name': "'app_friends_rel_+'"}),
            'scope': ('django.db.models.fields.CharField', [], {'blank': 'True', 'default': "''", 'max_length': '512'}),
            'user_id': ('django.db.models.fields.BigIntegerField', [], {'unique': 'True'}),
            'user_ptr': ('django.db.models.fields.related.OneToOneField', [], {'primary_key': 'True', 'to': "orm['auth.User']", 'unique': 'True'})
        },
        'facebook_auth.usertoken': {
            'Meta': {'object_name': 'UserToken'},
            'deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'expiration_date': ('django.db.models.fields.DateTimeField', [], {}),
            'granted_at': ('django.db.models.fields.DateTimeField', [], {'blank': 'True', 'auto_now_add': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'provider_user_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'token': ('django.db.models.fields.TextField', [], {'unique': 'True'})
        }
    }

    complete_apps = ['facebook_auth']



from django.contrib import admin

from facebook_auth import models


class UserTokenAdmin(admin.ModelAdmin):
    list_display = ('provider_user_id', 'expiration_date', 'deleted')
    list_filter = ('deleted',)
    search_fields = ('token', 'provider_user_id')


admin.site.register(models.UserToken, UserTokenAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='FacebookUser',
            fields=[
                ('user_ptr', models.OneToOneField(serialize=False, primary_key=True, to=settings.AUTH_USER_MODEL, auto_created=True)),
                ('user_id', models.BigIntegerField(unique=True)),
                ('scope', models.CharField(default='', max_length=512, blank=True)),
                ('app_friends', models.ManyToManyField(to='facebook_auth.FacebookUser')),
            ],
            options={
                'verbose_name': 'user',
                'abstract': False,
                'verbose_name_plural': 'users',
            },
            bases=('auth.user',),
        ),
        migrations.CreateModel(
            name='UserToken',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('provider_user_id', models.CharField(max_length=255)),
                ('token', models.TextField(unique=True)),
                ('granted_at', models.DateTimeField(auto_now_add=True)),
                ('expiration_date', models.DateTimeField(default=None, null=True, blank=True)),
                ('deleted', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'User token',
                'verbose_name_plural': 'User tokens',
            },
            bases=(models.Model,),
        ),
    ]



import json
try:
    from urllib.parse import urljoin
    from urllib.parse import urlencode
except ImportError:
    from urlparse import urljoin
    from urllib import urlencode

from django.conf import settings
from django.conf.urls import patterns
from django.conf.urls import url
from django.core import signing
from django.core.urlresolvers import reverse
from django.utils import encoding


class InvalidNextUrl(Exception):
    pass


class Next(object):
    salt = 'facebook_auth.urls.Next'

    def encode(self, data):
        data = self.dumps(data)
        return urlencode({'next': data})

    def decode(self, data):
        try:
            return self.loads(data)
        except signing.BadSignature:
            raise InvalidNextUrl()

    def dumps(self, obj):
        data = json.dumps(
            obj, separators=(',', ':'), sort_keys=True).encode('utf-8')
        base64d = signing.b64_encode(data)
        return signing.Signer(salt=self.salt).sign(base64d)

    def loads(self, s):
        base64d = encoding.force_bytes(
            signing.Signer(salt=self.salt).unsign(s))
        data = signing.b64_decode(base64d)
        return json.loads(data.decode('utf-8'))


def redirect_uri(next, close):
    return urljoin(
        settings.FACEBOOK_CANVAS_URL,
        reverse('facebook-auth-handler') + "?" +
        Next().encode({'next': next, 'close': close})
    )

urlpatterns = patterns('facebook_auth.views',
    url(r'^handler$', 'handler', name='facebook-auth-handler'),
)



# -*- coding: utf-8 -*-
from functools import wraps
import re
from uuid import uuid1

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

from django.conf import settings
from django import http

from django.contrib.auth import authenticate
from django.contrib.auth import login
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.utils.encoding import force_text
from django.utils import html


def get_auth_address(request, redirect_to, scope='', state=None):
    if not state:
        state = force_text(uuid1())
        auth_requests = request.session.get('auth_requests', {})
        if len(auth_requests) >= 20:
            auth_requests = {}
        auth_requests[state] = {
            'method': request.method,
            'POST': request.POST,
            'path': request.path,
            'scope': scope,
        }
        request.session['auth_requests'] = auth_requests
    else:
        scope = (request.session
                 .get('auth_requests', {})
                 .get(state, {})
                 .get('scope', ''))
    args = {
            'client_id': settings.FACEBOOK_APP_ID,
            'redirect_uri': redirect_to,
            'scope': scope,
            'state': state,
    }
    return 'https://www.facebook.com/dialog/oauth?' + urlencode(args)


def use_fallback(get):
    return ('access_denied' == get.get('error', '') and
            'user_denied' == get.get('error_reason', ''))


def accept_login(fallback_template=None, scope=''):
    def decorator(fun):
        @wraps(fun)
        def res(request, *args, **kwargs):
            state = request.GET.get('state', None)
            code = request.GET.get('code', None)
            if state and code:
                old_request = request.session.get('auth_requests', {}).get(
                    state, None)
                if old_request and old_request['path'] == request.path:
                    request.method = old_request['method']
                    request.POST = old_request['POST']
                    del request.session['auth_requests'][state]
                    request.session.modified = True
                    user = authenticate(
                        code=code,
                        redirect_uri=request.build_absolute_uri(request.path))
                    if user:
                        login(request, user)
                if request.method != 'POST':
                    return HttpResponseRedirect(
                        request.build_absolute_uri(request.path))
            if fallback_template and state and use_fallback(request.GET):
                url = get_auth_address(request, request.build_absolute_uri(
                    request.path), state=state)
                return TemplateResponse(request, fallback_template,
                    context={'url': url})
            return fun(request, *args, **kwargs)
        return res
    return decorator


def login_required(scope='', return_view_content=False):
    def decorator(fun):
        @wraps(fun)
        def res(request, *args, **kwargs):
            if request.user.is_authenticated():
                return fun(request, *args, **kwargs)
            else:
                url = get_auth_address(request, request.build_absolute_uri(
                    request.path), scope)
                if return_view_content:
                    response = fun(request, *args, **kwargs)
                else:
                    response = http.HttpResponse(
                        "<html><head><title></title></head><body></body>"
                        "</html>")
                redirect = ("<script>window.top.location=\"%(url)s\";"
                            "</script>" % dict(url=html.escapejs(url)))
                response.content = re.sub(
                    "<body(.*?)>", "<body\\1>" + redirect, response.content)
                return response
        return res
    return decorator



#!/usr/bin/env python
from setuptools import setup


def read(name):
    from os import path
    return open(path.join(path.dirname(__file__), name)).read()

setup(
    name='django-facebook-auth',
    version='3.6.1',
    description="Authorisation app for Facebook API.",
    long_description=read("README.rst"),
    maintainer="Tomasz Wysocki",
    maintainer_email="tomasz@wysocki.info",

    install_requires=(
        'celery',
        'Django<1.8',
        'facepy>=1.0.3',
    ),

    packages=[
        'facebook_auth',
        'facebook_auth.migrations',
        'facebook_auth.management',
        'facebook_auth.management.commands',
        'facebook_auth.south_migrations',
    ],
)



from django.core.management.base import BaseCommand

from facebook_auth import models


class Command(BaseCommand):
    def handle(self, *args, **options):
        users = (models.UserToken.objects.distinct('provider_user_id')
                 .values_list('provider_user_id', flat=True))
        for user in users:
            models.debug_all_tokens_for_user.delay(user)
            self.stdout.write('Debugging user "%s"' % user)



from django.conf import settings
import facepy

from facebook_auth.graph_api import ObservableGraphAPI

GRAPH_MAX_TRIES = 3


def get_from_graph_api(graphAPI, query):
    for i in range(GRAPH_MAX_TRIES):
        try:
            return graphAPI.get(query)
        except facepy.FacepyError as e:
            if i == GRAPH_MAX_TRIES - 1 or getattr(e, 'code', None) != 1:
                raise


def get_application_graph():
    token = facepy.utils.get_application_access_token(
            settings.FACEBOOK_APP_ID, settings.FACEBOOK_APP_SECRET)
    return ObservableGraphAPI(token)
